@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2021 OARC, Inc.
+ * Copyright 2019-2023 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -28,7 +28,7 @@
 
 #include "net.h"
 #include "edns.h"
-#include "parse_uri.h"
+#include "ext/parse_uri.h"
 #include "log.h"
 #include "strerror.h"
 #include "util.h"
@@ -118,6 +118,10 @@ struct perf__doh_socket {
 
     char   recvbuf[TCP_RECV_BUF_SIZE];
     size_t recv_at;
+
+    size_t       num_queries_per_conn, nqpc_timeout;
+    unsigned int nqpc_sent, nqpc_recv;
+    uint64_t     nqpc_ts;
 };
 
 static pthread_mutex_t            _nghttp2_lock      = PTHREAD_MUTEX_INITIALIZER;
@@ -157,6 +161,10 @@ static void perf__doh_connect(struct perf_net_socket* sock)
 {
     int ret;
 
+    self->nqpc_sent = 0;
+    ck_pr_store_uint(&self->nqpc_recv, 0);
+    self->nqpc_ts = 0;
+
     nghttp2_session_del(self->http2.session);
     ret = nghttp2_session_client_new2(&self->http2.session, _nghttp2_callbacks, sock, _nghttp2_option);
     if (ret < 0) {
@@ -316,6 +324,8 @@ static ssize_t perf__doh_recv(struct perf_net_socket* sock, void* buf, size_t le
                 return -1;
             case SSL_ERROR_SYSCALL:
                 switch (errno) {
+                case EBADF:
+                    // treat this as a retry, can happen if sendto is reconnecting
                 case ECONNREFUSED:
                 case ECONNRESET:
                 case ENOTCONN:
@@ -394,6 +404,9 @@ static ssize_t perf__doh_recv(struct perf_net_socket* sock, void* buf, size_t le
 
         // self->have_more = false; TODO
         PERF_UNLOCK(&self->lock);
+        if (self->num_queries_per_conn) {
+            ck_pr_inc_uint(&self->nqpc_recv);
+        }
         return len;
     }
 
@@ -586,6 +599,8 @@ static ssize_t perf__doh_sendto(struct perf_net_socket* sock, uint16_t qid, cons
     }
     PERF_UNLOCK(&self->lock);
 
+    self->nqpc_sent++;
+
     return len;
 }
 
@@ -628,6 +643,21 @@ static int perf__doh_sockready(struct perf_net_socket* sock, int pipe_fd, int64_
             }
             self->is_sending = false;
             sent             = true;
+            self->nqpc_sent++;
+        }
+        if (self->num_queries_per_conn && self->nqpc_sent >= self->num_queries_per_conn) {
+            if (!self->nqpc_ts) {
+                self->nqpc_ts = perf_get_time() + self->nqpc_timeout;
+            }
+            unsigned int r = ck_pr_load_uint(&self->nqpc_recv);
+            if (r >= self->nqpc_sent || perf_get_time() > self->nqpc_ts) {
+                self->do_reconnect = true;
+            }
+            PERF_UNLOCK(&self->lock);
+            if (sent && sock->sent) {
+                sock->sent(sock, self->qid);
+            }
+            return 0;
         }
         PERF_UNLOCK(&self->lock);
         if (sent && sock->sent) {
@@ -898,6 +928,12 @@ static int select_next_proto_cb(SSL* ssl, unsigned char** out,
 }
 #endif /* !OPENSSL_NO_NEXTPROTONEG */
 
+static void perf__doh_num_queries_per_conn(struct perf_net_socket* sock, size_t num_queries_per_conn, size_t timeout)
+{
+    self->num_queries_per_conn = num_queries_per_conn;
+    self->nqpc_timeout         = timeout;
+}
+
 struct perf_net_socket* perf_net_doh_opensocket(const perf_sockaddr_t* server, const perf_sockaddr_t* local, size_t bufsize, void* data, perf_net_sent_cb_t sent, perf_net_event_cb_t event)
 {
     struct perf__doh_socket* tmp  = calloc(1, sizeof(struct perf__doh_socket)); // clang scan-build
@@ -915,6 +951,8 @@ struct perf_net_socket* perf_net_doh_opensocket(const perf_sockaddr_t* server, c
     sock->sockready = perf__doh_sockready;
     sock->have_more = perf__doh_have_more;
 
+    sock->num_queries_per_conn = perf__doh_num_queries_per_conn;
+
     sock->data  = data;
     sock->sent  = sent;
     sock->event = event;
